/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "finddialog.h"
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qhbox.h>
#include <qhgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qtextbrowser.h>
#include <qvbox.h>


FindDialog::FindDialog(QTextBrowser *browser, QWidget *parent, const char *name)
    : QDialog(parent, name)
{
    setCaption(trUtf8("Find"));

    QVBoxLayout *layout = new QVBoxLayout(this, 11, 6);
    layout->setAutoAdd(true);
    
    QHBox *findBox = new QHBox(this);
    QLabel *findLabel = new QLabel(trUtf8("&Find:"), findBox);
    findCombo = new QComboBox(true, findBox);
    findCombo->setDuplicatesEnabled(false);
    findCombo->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

    QHGroupBox *groupBox = new QHGroupBox(trUtf8("&Options"), this);
    csBox = new QCheckBox(trUtf8("&Case sensitive"), groupBox);
    backBox = new QCheckBox(trUtf8("Find &backwards"), groupBox);

    QHBox *buttonBox = new QHBox(this);
    QPushButton *findButton = new QPushButton(trUtf8("&Find"), buttonBox);
    QPushButton *closeButton = new QPushButton(trUtf8("&Close"), buttonBox);

    connect( findButton, SIGNAL(clicked()),
             this, SLOT(findClicked()) );
    connect( closeButton, SIGNAL(clicked()),
             this, SLOT(reject()) );

    findLabel->setBuddy(findCombo);
    findButton->setDefault(true);
    continuing = false;
    htmlView = browser;
}


FindDialog::~FindDialog()
{}


void FindDialog::showEvent(QShowEvent*)
{
    continuing = false;
    findCombo->setFocus();
    findCombo->lineEdit()->selectAll();
}


void FindDialog::findClicked()
{
    QString text = findCombo->currentText();
    bool cs = csBox->isChecked();
    bool back = backBox->isChecked();
    
    bool success;
    if (continuing)
	success = htmlView->find(text, cs, false, !back);
    else {
        int para=0, index=0;
	success = htmlView->find(text, cs, false, !back, &para, &index);
    }

    if (!success) {
        QMessageBox::information(this, trUtf8("Find"),
                                 continuing? trUtf8("No further match") : trUtf8("No match"));
        continuing = false;
    } else
        continuing = true;
}
