/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "../config.h"

#include <qdir.h>
#include <qdragobject.h>
#include <qfileinfo.h>
#include <qiconset.h>
#include <qsettings.h>
#include <qstringlist.h>


QPixmap Global::pixmap(const QString &name)
{
    const QMimeSource *mime = QMimeSourceFactory::defaultFactory()->data(name);
    QPixmap pix;
    QImageDrag::decode(mime, pix);
    return pix;
}


QIconSet Global::iconSet(const QString &name)
{
    return QIconSet(pixmap(name));
}


QString Global::resource(const QString &relpath)
{
    return QString::fromLocal8Bit(PREFIX) + _("/") + relpath;
}


QStringList Global::resources(const QString &relpath)
{
    QStringList res;
    
    QDir d(QString::fromLocal8Bit(PREFIX) + _("/") + relpath);
    const QFileInfoList *l = d.entryInfoList();

    if (!l)
        return QStringList();
    
    QFileInfoListIterator it(*l);
    ++it; ++it;
    for (; it.current(); ++it) {
        QString name = it.current()->absFilePath();
        // qDebug("File: %s", name.latin1());
        res.append(name);
    }

    return res;
}


static QSettings *globalSettings = 0;


QSettings *Global::settings()
{
    if (!globalSettings)
        globalSettings = new QSettings;
    
    return globalSettings;
}


void Global::cleanup()
{
    delete globalSettings;
}
