/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef _INDEXWIDGET_H_
#define _INDEXWIDGET_H_

#include <qdom.h>
#include <qlist.h>
#include <qmap.h>
#include <qsplitter.h>

class QLineEdit;
class QListBox;
class QListBoxItem;
class QTabWidget;
class IndexFile;


class IndexWidget : public QSplitter
{
    Q_OBJECT
    
public:
    IndexWidget(QTabWidget *parent, const char *name);
    ~IndexWidget();

signals:
    void gotoFile(const QString &url);
    void highlighted(const QString &url);

private slots:
    void textChanged(const QString &text);
    void returnPressed();
    void indexItemSelected(QListBoxItem *item);
    void onIndexItem(QListBoxItem *item);
    void onIndexViewport();
    void topicItemSelected(QListBoxItem *item);
    void onTopicItem(QListBoxItem *item);
    void onTopicViewport();
    void showTerm(const QString &name);

private:
    void showEvent(QShowEvent *e);
    bool eventFilter(QObject *o, QEvent *e);
    
    void readIndexFromFile(const QString &fileName);
    void readEntryList(const QDomElement &el, char fileno);

    QTabWidget *tabParent;
    QLineEdit *termEdit;
    QListBox *indexListBox;
    QListBox *topicListBox;
    QList<IndexFile> indexFiles;
    QMap<QString,QListBoxItem*> indexMap;
};

#endif // _INDEXWIDGET_H_

