/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "mainwindow.h"
#include "global.h"
#include <qapplication.h>
#include <qmetaobject.h>
#include <qobjectlist.h>
#include <private/qucomextra_p.h>
#include <pwd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

extern Time qt_x_time;


class DoxApplication : public QApplication
{
public:
    DoxApplication(int &argc, char **argv)
        : QApplication(argc, argv)
    {
        selectionOwner = new QWidget(0);

        char hostName[512];
        struct passwd *pw = ::getpwuid(::getuid());
        const char *userName = pw? pw->pw_name : "";
        (void) ::gethostname(hostName, sizeof hostName);
 
        QCString propName = name();
        propName += "_SERVER_RUNNING_";
        propName += hostName;
        propName += "_";
        propName += userName;
        runningAtom = XInternAtom(qt_xdisplay(), propName, False);
        propName = name();
        propName += "_SERVER_COMMAND_";
        propName += hostName;
        propName += "_";
        propName += userName;
        commandAtom = XInternAtom(qt_xdisplay(), propName, False);

        if (argc == 3 && strcmp(argv[1], "-remote") == 0) {
            isclient = true;
            if (XGetSelectionOwner(qt_xdisplay(), runningAtom) == None) {
                fprintf(stderr, "Server does not seem to run currently.\n");
                return;
            }
            XChangeProperty(qt_xdisplay(), qt_xrootwin(), commandAtom, XA_STRING, 8,
                            PropModeReplace, (unsigned char *)argv[2], strlen(argv[2]));
        } else {
            isclient = false;
            XSetSelectionOwner(qt_xdisplay(), runningAtom,
                               selectionOwner->winId(), qt_x_time);
        }
    }
    
    ~DoxApplication()
    {
        delete selectionOwner;
    }

    bool isClient() const
    {
        return isclient;
    }

    bool x11EventFilter(XEvent *e)
    {
        if (e->xproperty.window != qt_xrootwin())
            return false;

        if (e->xproperty.atom == commandAtom
            && e->xproperty.state == PropertyNewValue) {
            Atom actual_type;
            int actual_format;
            unsigned long nitems, bytes_after;
            unsigned char *prop;
            
            if (XGetWindowProperty(qt_xdisplay(), qt_xrootwin(),
                                   commandAtom, 0, INT_MAX, True, XA_STRING,
                                   &actual_type, &actual_format, &nitems,
                                   &bytes_after, &prop) != Success || actual_format != 8)
                return true;

            processClientCommand((char*)prop);
            XFree(prop);
            return true;
        }
        
        return false;
    }

    static void processClientCommand(QCString command)
    {
        int pos1 = command.find(".");
        if (pos1 == -1) {
            qDebug("No . in command");
            return;
        }
        int pos2 = command.find("(", pos1+1);
        if (pos2 == -1) {
            qDebug("No ( in command");
            return;
        }
        QCString objectName = QCString("REMOTE:") + command.left(pos1);
        QCString methodName = command.mid(pos1+1, pos2-pos1-1);
        QStringList args = QStringList::split(_(","), command.mid(pos2+1, command.length()-pos2-2));
        QObject *o = findObject(QObject::objectTrees(), objectName);
        if (!o) {
            qDebug("No object with this name found");
            return;
        }
        int numSlots = o->metaObject()->numSlots(true);
        for (int i = 0; i < numSlots; ++i) {
            const QMetaData *md = o->metaObject()->slot(i, true);
            if (strcmp(md->method->name, methodName.data()) == 0
                && md->method->count == (int)args.count()) {
                QUObject *u = new QUObject[args.count()+1];
                for (int j = 0; j < (int)args.count(); ++j)
                    static_QUType_QString.set(u+j+1, args[j]);
                o->qt_invoke(i, u);
                delete[] u;
                return;
            }
        }
        qDebug("No method with this name and number of arguments");
    }

    static QObject *findObject(const QObjectList *l, const QCString &name)
    {
        if (!l)
            return 0;
        for (QObjectListIt it(*l); it.current(); ++it) {
            if (strcmp(it.current()->name(), name) == 0)
                return it.current();
            QObject *o = findObject(it.current()->children(), name);
            if (o)
                return o;
        }
        return 0;
    }

private:
    Atom runningAtom;
    Atom commandAtom;
    QWidget *selectionOwner;
    bool isclient;
};


int main(int argc, char **argv)
{
    if (!getenv("MANPATH"))
        putenv("MANPATH=/usr/local/man:/usr/X11R6/man:"
               "/usr/share/man:/usr/man");
    if (!getenv("INFOPATH"))
        putenv("INFOPATH=/usr/local/info:/usr/share/texmf/teTeX/info:"
               "/usr/share/xemacs/info:/usr/share/info:/usr/info");
    
    DoxApplication app(argc, argv);
    if (app.isClient()) {
        app.unlock();
        return 0;
    }
    MainWindow *w = new MainWindow(0, "REMOTE:MainWindow");
    app.setMainWidget(w);
    w->show();
    
    int res = app.exec();
    Global::cleanup();
    return res;
}
