/***************************************************************************
 *   Copyright (C) 2002 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "mainwindow.h"
#include "contentswidget.h"
#include "indexwidget.h"
#include "searchwidget.h"
#include "richtextview.h"
#include "finddialog.h"
#include "settingsdialog.h"
#include "global.h"

#include <qaction.h>
#include <qapplication.h>
#include <qdockwindow.h>
#include <qdragobject.h>
#include <qiconset.h>
#include <qinputdialog.h>
#include <qlineedit.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qpopupmenu.h>
#include <qprinter.h>
#include <qprocess.h>
#include <qregexp.h>
#include <qsettings.h>
#include <qsimplerichtext.h>
#include <qstatusbar.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qtoolbar.h>
#include <qtooltip.h>


MainWindow::MainWindow(QWidget *parent, const char *name)
    : QMainWindow(parent, name, WType_TopLevel | WDestructiveClose)
{
    setCaption(_("Dox"));

    fileOpenAction = new QAction(this, "fileOpenAction");
    fileOpenAction->setIconSet(Global::iconSet(_("fileopen")));
    fileOpenAction->setText(trUtf8("Open"));
    fileOpenAction->setMenuText(trUtf8("&Open..."));
    fileOpenAction->setAccel(QKeySequence(trUtf8("Ctrl+O")));
    filePrintAction = new QAction(this, "filePrintAction");
    filePrintAction->setIconSet(Global::iconSet(_("fileprint")));
    filePrintAction->setText(trUtf8("Print"));
    filePrintAction->setMenuText(trUtf8("&Print..."));
    filePrintAction->setAccel(QKeySequence(trUtf8("Ctrl+P")));
    fileIndexAction = new QAction(this, "fileIndexAction");
    fileIndexAction->setText(trUtf8("Create fulltext search index"));
    fileIndexAction->setMenuText(trUtf8("Create &index"));
    fileExitAction = new QAction(this, "fileExitAction");
    fileExitAction->setText(trUtf8("Exit"));
    fileExitAction->setMenuText(trUtf8("E&xit"));
    fileExitAction->setAccel(QKeySequence(trUtf8("Ctrl+Q")));
    editCopyAction = new QAction(this, "editCopyAction");
    editCopyAction->setIconSet(Global::iconSet(_("editcopy")));
    editCopyAction->setText(trUtf8("Copy"));
    editCopyAction->setMenuText(trUtf8("C&opy"));
    editCopyAction->setAccel(QKeySequence(trUtf8("Ctrl+C")));
    editFindAction = new QAction(this, "editFindAction");
    editFindAction->setIconSet(Global::iconSet(_("find")));
    editFindAction->setText(trUtf8("Find"));
    editFindAction->setMenuText(trUtf8("&Find..."));
    editFindAction->setAccel(QKeySequence(trUtf8("Ctrl+F")));
    editIsearchAction = new QAction(this, "editIsearchAction");
    editIsearchAction->setText(trUtf8("ISearch"));
    editIsearchAction->setMenuText(trUtf8("&ISearch..."));
    editIsearchAction->setAccel(QKeySequence(trUtf8("Ctrl+I")));
    editSettingsAction = new QAction(this, "editSettingsAction");
    editSettingsAction->setText(trUtf8("Settings"));
    editSettingsAction->setMenuText(trUtf8("&Settings..."));
    viewBackwardAction = new QAction(this, "viewBackwardAction");
    viewBackwardAction->setIconSet(Global::iconSet(_("back")));
    viewBackwardAction->setText(trUtf8("Back"));
    viewBackwardAction->setAccel(QKeySequence(trUtf8("Ctrl+Left")));
    viewForwardAction = new QAction(this, "viewforwardAction");
    viewForwardAction->setIconSet(Global::iconSet(_("forward")));
    viewForwardAction->setText(trUtf8("Forward"));
    viewForwardAction->setAccel(QKeySequence(trUtf8("Ctrl+Right")));
    viewZoomInAction = new QAction(this, "viewzoominAction");
    viewZoomInAction->setIconSet(Global::iconSet(_("zoomin")));
    viewZoomInAction->setText(trUtf8("Zoom in"));
    viewZoomInAction->setMenuText(trUtf8("Zoom &in"));
    viewZoomOutAction = new QAction(this, "viewzoomoutAction");
    viewZoomOutAction->setIconSet(Global::iconSet(_("zoomout")));
    viewZoomOutAction->setText(trUtf8("Zoom out"));
    viewZoomOutAction->setMenuText(trUtf8("Zoom &out"));
    bookmarkAddAction = new QAction(this, "bookmarkAddAction");
    bookmarkAddAction->setText(trUtf8("Add Bookmark"));
    bookmarkAddAction->setMenuText(trUtf8("&Add Bookmark"));
    bookmarkRemoveAction = new QAction(this, "bookmarkRemoveAction");
    bookmarkRemoveAction->setText(trUtf8("Remove Bookmark"));
    bookmarkRemoveAction->setMenuText(trUtf8("&Remove Bookmark"));
    helpAboutAction = new QAction(this, "helpAboutAction");
    helpAboutAction->setText(trUtf8("About"));
    helpAboutAction->setMenuText(trUtf8("&About..."));

    QToolBar *toolBar = new QToolBar(trUtf8("Tools"), this, DockTop); 
    viewBackwardAction->addTo(toolBar);
    viewForwardAction->addTo(toolBar);
    editCopyAction->addTo(toolBar);
    toolBar->addSeparator();
    editFindAction->addTo(toolBar);
    isearchEdit = new QLineEdit(toolBar);
    isearchEdit->installEventFilter(this);
    QToolTip::add(isearchEdit, editIsearchAction->toolTip());

    QMenuBar *menubar = new QMenuBar(this);

    QPopupMenu *fileMenu = new QPopupMenu(this); 
    fileOpenAction->addTo(fileMenu);
    fileMenu->insertSeparator();
    filePrintAction->addTo(fileMenu);
    fileIndexAction->addTo(fileMenu);
    fileMenu->insertSeparator();
    fileExitAction->addTo(fileMenu);
    menubar->insertItem(trUtf8("&File"), fileMenu);

    QPopupMenu *editMenu = new QPopupMenu(this);
    editMenu->insertSeparator();
    editMenu->insertSeparator();
    editCopyAction->addTo(editMenu);
    editFindAction->addTo(editMenu);
    editIsearchAction->addTo(editMenu);
    editSettingsAction->addTo(editMenu);
    menubar->insertItem(trUtf8("&Edit"), editMenu);

    QPopupMenu *viewMenu = new QPopupMenu(this);
    viewBackwardAction->addTo(viewMenu);
    viewForwardAction->addTo(viewMenu);
    viewMenu->insertSeparator();
    viewZoomInAction->addTo(viewMenu);
    viewZoomOutAction->addTo(viewMenu);
    menubar->insertItem(trUtf8("&View"), viewMenu);
    
    historyMenu = new QPopupMenu(this);
    menubar->insertItem(trUtf8("&History"), historyMenu);

    bookmarkMenu = new QPopupMenu(this);
    menubar->insertItem(trUtf8("&Bookmarks"), bookmarkMenu);
    
    QPopupMenu *helpMenu = new QPopupMenu(this);
    helpMenu->insertSeparator();
    helpAboutAction->addTo(helpMenu);
    menubar->insertItem(trUtf8("&Help"), helpMenu);

    dockWindow = new QDockWindow;
    QTabWidget *tabWidget = new QTabWidget(dockWindow);
    dockWindow->setWidget(tabWidget);
    dockWindow->setCloseMode(QDockWindow::Never);
    dockWindow->setResizeEnabled(true);
    dockWindow->setCaption(trUtf8("Sidebar"));
    addDockWindow(dockWindow, DockLeft);
    
    ContentsWidget *contentsWidget = new ContentsWidget(tabWidget, "REMOTE:ContentsWidget");
    tabWidget->addTab(contentsWidget, trUtf8("&Contents"));

    IndexWidget *indexWidget = new IndexWidget(tabWidget, "REMOTE:IndexWidget");
    tabWidget->addTab(indexWidget, trUtf8("&Index"));

    SearchWidget *searchWidget = new SearchWidget(tabWidget, "REMOTE:SearchWidget");
    tabWidget->addTab(searchWidget, trUtf8("&Search"));
    
    htmlView = new RichtextView(this, "REMOTE:RichtextView");
    setCentralWidget(htmlView);

    findDialog = 0;
    
    connect( fileOpenAction, SIGNAL(activated()),
             this, SLOT(fileOpen()) );
    connect( filePrintAction, SIGNAL(activated()),
             this, SLOT(filePrint()) );
    connect( fileIndexAction, SIGNAL(activated()),
             this, SLOT(fileIndex()) );
    connect( fileExitAction, SIGNAL(activated()),
             qApp, SLOT(closeAllWindows()) );
    connect( editCopyAction, SIGNAL(activated()),
             htmlView, SLOT(copy()) );
    connect( editFindAction, SIGNAL(activated()),
             this, SLOT(editFind()) );
    connect( editIsearchAction, SIGNAL(activated()),
             this, SLOT(editIsearch()) );
    connect( editSettingsAction, SIGNAL(activated()),
             this, SLOT(editSettings()) );
    connect( bookmarkAddAction, SIGNAL(activated()),
             this, SLOT(bookmarkAdd()) );
    connect( bookmarkRemoveAction, SIGNAL(activated()),
             this, SLOT(bookmarkRemove()) );
    connect( helpAboutAction, SIGNAL(activated()),
             this, SLOT(helpAbout()) );
    connect( viewBackwardAction, SIGNAL(activated()),
             htmlView, SLOT(backward()) );
    connect( viewForwardAction, SIGNAL(activated()),
             htmlView, SLOT(forward()) );
    connect( viewZoomInAction, SIGNAL(activated()),
             htmlView, SLOT(zoomIn()) );
    connect( viewZoomOutAction, SIGNAL(activated()),
             htmlView, SLOT(zoomOut()) );
    connect( isearchEdit, SIGNAL(textChanged(const QString &)),
	     this, SLOT(isearchChanged()) );
    connect( isearchEdit, SIGNAL(returnPressed()),
	     this, SLOT(isearchReturnPressed()) );
    
    connect( contentsWidget, SIGNAL(gotoFile(const QString &)),
             this, SLOT(gotoFile(const QString &)) );
    connect( contentsWidget, SIGNAL(highlighted(const QString &)),
             statusBar(), SLOT(message(const QString &)) );
    connect( indexWidget, SIGNAL(gotoFile(const QString &)),
             this, SLOT(gotoFile(const QString &)) );
    connect( indexWidget, SIGNAL(highlighted(const QString &)),
             statusBar(), SLOT(message(const QString &)) );
    connect( searchWidget, SIGNAL(gotoFile(const QString &)),
             this, SLOT(gotoFile(const QString &)) );

    connect( historyMenu, SIGNAL(highlighted(int)),
             this, SLOT(historyHighlighted(int)) );
    connect( historyMenu, SIGNAL(activated(int)),
             this, SLOT(historyActivated(int)) );
    connect( bookmarkMenu, SIGNAL(highlighted(int)),
             this, SLOT(bookmarkHighlighted(int)) );
    connect( bookmarkMenu, SIGNAL(activated(int)),
             this, SLOT(bookmarkActivated(int)) );

    viewBackwardAction->setEnabled(false);
    viewForwardAction->setEnabled(false);
    connect( htmlView, SIGNAL(backwardAvailable(bool)),
             viewBackwardAction, SLOT(setEnabled(bool)) );
    connect( htmlView, SIGNAL(forwardAvailable(bool)),
             viewForwardAction, SLOT(setEnabled(bool)) );
    connect( htmlView, SIGNAL(highlighted(const QString &)),
             statusBar(), SLOT(message(const QString &)) );
    connect( htmlView, SIGNAL(wentToFile(const QString &)),
             this, SLOT(wentToFile(const QString &)) );

    QSettings *settings = Global::settings();
    int width = settings->readNumEntry(_("/Dox/MainWindow/Width"), sizeHint().width());
    int height = settings->readNumEntry(_("/Dox/MainWindow/Height"), sizeHint().height());
    resize(width, height);
    
    QString dockconf = settings->readEntry(_("/Dox/MainWindow/DockConfig"), _(""));
    if (!dockconf.isEmpty()) {
        dockconf.replace(QRegExp(_("\\$")), _("\n"));
        QTextStream stream(&dockconf, IO_ReadOnly);
        stream >> (*this);
    }

    QStringList historyTitles = settings->readListEntry(_("/Dox/MainWindow/HistoryTitles"));
    QStringList historyUrls = settings->readListEntry(_("/Dox/MainWindow/HistoryUrls"));
    QStringList::ConstIterator hti = historyTitles.begin();
    QStringList::ConstIterator hui = historyUrls.begin();
    while (hti != historyTitles.end() && hui != historyUrls.end()) {
        historyEntries.append(Bookmark(*hti, *hui));
        ++hti;
        ++hui;
    }
    updateHistoryMenu();

    QStringList bookmarkTitles = settings->readListEntry(_("/Dox/MainWindow/BookmarkTitles"));
    QStringList bookmarkUrls = settings->readListEntry(_("/Dox/MainWindow/BookmarkUrls"));
    QStringList::ConstIterator bti = bookmarkTitles.begin();
    QStringList::ConstIterator bui = bookmarkUrls.begin();
    while (bti != bookmarkTitles.end() && bui != bookmarkUrls.end()) {
        bookmarks.append(Bookmark(*bti, *bui));
        ++bti;
        ++bui;
    }
    updateBookmarkMenu();
}


MainWindow::~MainWindow()
{
    QString dockconf;
    QTextStream stream(&dockconf, IO_WriteOnly);
    stream << (*this);
    dockconf.replace(QRegExp(_("\n")), _("$"));
    
    QSettings *settings = Global::settings();
    settings->writeEntry(_("/Dox/MainWindow/Width"), width());
    settings->writeEntry(_("/Dox/MainWindow/Height"), height());
    settings->writeEntry(_("/Dox/MainWindow/DockConfig"), dockconf);

    QStringList historyTitles, historyUrls;
    QValueList<Bookmark>::ConstIterator hi;
    for (hi = historyEntries.begin(); hi != historyEntries.end(); ++hi) {
        historyTitles.append((*hi).first);
        historyUrls.append((*hi).second);
    }
    settings->writeEntry(_("/Dox/MainWindow/HistoryTitles"), historyTitles);
    settings->writeEntry(_("/Dox/MainWindow/HistoryUrls"), historyUrls);

    QStringList bookmarkTitles, bookmarkUrls;
    QValueList<Bookmark>::ConstIterator bmi;
    for (bmi = bookmarks.begin(); bmi != bookmarks.end(); ++bmi) {
        bookmarkTitles.append((*bmi).first);
        bookmarkUrls.append((*bmi).second);
    }
    settings->writeEntry(_("/Dox/MainWindow/BookmarkTitles"), bookmarkTitles);
    settings->writeEntry(_("/Dox/MainWindow/BookmarkUrls"), bookmarkUrls);
}


bool MainWindow::eventFilter(QObject *o, QEvent *e)
{
    if (e->type() == QEvent::KeyPress && o == isearchEdit) {
        if (static_cast<QKeyEvent*>(e)->key() == Key_Escape) {
            htmlView->setFocus();
            return true;
        }
    }

    return QMainWindow::eventFilter(o, e);
}


void MainWindow::setActiveWindow()
{
    // QWidget::raise() doesn't give focus to the window by itself.
    QMainWindow::raise();
    QMainWindow::setActiveWindow();
}


void MainWindow::gotoFile(const QString &url)
{
    htmlView->setSource(url);
    htmlView->setFocus();
}


void MainWindow::wentToFile(const QString &url)
{
    QString title = htmlView->documentTitle();
    if (title.isNull())
        title = url;
    setCaption(trUtf8("Dox: %1").arg(url));

    historyEntries.append(Bookmark(title, url));
    if (historyEntries.count() > 20)
        historyEntries.remove(historyEntries.begin());
    updateHistoryMenu();
}


void MainWindow::fileOpen()
{
    QString url = QInputDialog::getText(trUtf8("Open"), trUtf8("Open file name:"),
                                        QLineEdit::Normal, QString::null, 0, this);
    if (!url.isEmpty())
        gotoFile(url);
}


void MainWindow::filePrint()
{
    // from the textedit example
    QPrinter printer;
    printer.setFullPage(true);
    if (printer.setup(this)) {
        QPainter p(&printer);
        // Check that there is a valid device to print to.
        if (!p.device())
            return;
        QPaintDeviceMetrics metrics(p.device());
        int dpix = metrics.logicalDpiX();
        int dpiy = metrics.logicalDpiY();
        const int margin = 72; // pt
        QRect body(margin * dpix / 72, margin * dpiy / 72,
                   metrics.width() - margin * dpix / 72 * 2,
                   metrics.height() - margin * dpiy / 72 * 2);
        QFont font(_("times"), 10);
        QSimpleRichText richText(htmlView->text(), font,
                                 htmlView->context(),
                                 htmlView->styleSheet(),
                                 htmlView->mimeSourceFactory(),
                                 body.height());
        richText.setWidth(&p, body.width());
        QRect view(body);
        int page = 1;
        do {
            richText.draw(&p, body.left(), body.top(), view, colorGroup());
            view.moveBy(0, body.height());
            p.translate(0 , -body.height());
            p.setFont(font);
            p.drawText(view.right() - p.fontMetrics().width(QString::number(page)),
                       view.bottom() + p.fontMetrics().ascent() + 5,
                       QString::number(page));
            if (view.top()  >= richText.height())
                break;
            printer.newPage();
            page++;
        } while (true);
    }
}


void MainWindow::fileIndex()
{
    QProcess proc(_("dox-htdig"));
    proc.start();
}


void MainWindow::editFind()
{
    if (!findDialog)
        findDialog = new FindDialog(htmlView, this, "find dialog");
    
    findDialog->show();
}


void MainWindow::editIsearch()
{
    isearchEdit->setFocus();
    isearchEdit->selectAll();
}


void MainWindow::isearchChanged()
{
    QString text = isearchEdit->text();
    int para=0, index=0;
    if (!htmlView->find(text, false, false, true, &para, &index))
        statusBar()->message(trUtf8("No match"));
}


void MainWindow::isearchReturnPressed()
{
    QString text = isearchEdit->text();
    if (!htmlView->find(text, false, false, true))
        statusBar()->message(trUtf8("No further match"));
}


void MainWindow::editSettings()
{
    SettingsDialog dlg;
    (void) dlg.exec();
}


void MainWindow::updateHistoryMenu()
{
    historyMenu->clear();
    
    int id = 0;
    QValueList<Bookmark>::ConstIterator it;
    for (it = historyEntries.begin(); it != historyEntries.end(); ++it)
        historyMenu->insertItem((*it).first, ++id);
}


void MainWindow::historyHighlighted(int id)
{
    QString url = historyEntries[id-1].second;
    statusBar()->message(url);
}


void MainWindow::historyActivated(int id)
{
    QString url = historyEntries[id-1].second;
    gotoFile(url);
}


void MainWindow::updateBookmarkMenu()
{
    bookmarkMenu->clear();
    bookmarkAddAction->addTo(bookmarkMenu);
    bookmarkRemoveAction->addTo(bookmarkMenu);
    bookmarkMenu->insertSeparator();
    
    int id = 0;
    QValueList<Bookmark>::ConstIterator it;
    for (it = bookmarks.begin(); it != bookmarks.end(); ++it)
        bookmarkMenu->insertItem((*it).first, ++id);
}


void MainWindow::bookmarkHighlighted(int id)
{
    if (id < 0)
        return;
    QString url = bookmarks[id-1].second;
    statusBar()->message(url);
}


void MainWindow::bookmarkActivated(int id)
{
    if (id < 0)
        return;
    QString url = bookmarks[id-1].second;
    gotoFile(url);
}


void MainWindow::bookmarkAdd()
{
    QString title = htmlView->documentTitle();
    if (title.isNull())
        title = htmlView->source();
    QString url = htmlView->source();
    if (url.isNull())
        return;
    bookmarks.append(Bookmark(title, url));

    updateBookmarkMenu();
}


void MainWindow::bookmarkRemove()
{
    QString url = htmlView->source();
    QValueList<Bookmark>::Iterator it;
    for (it = bookmarks.begin(); it != bookmarks.end(); ++it)
        if ((*it).second == url) {
            bookmarks.remove(it);
            updateBookmarkMenu();
            break;
        }
}


void MainWindow::helpAbout()
{
    QMessageBox::about(this, trUtf8("Dox"),
                       trUtf8("Dox 1.1\n"
                              "Copyright (C) 1999-2002 Bernd Gehrmann\n"
                              "\n"
                              "This program is free software; you can redistribute it and/or modify\n"
                              "it under the terms of the GNU General Public License, version 2\n"
                              "as published by the Free Software Foundation."));
}
