/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include <qmainwindow.h>
#include <qpair.h>
#include <qvaluelist.h>

class QAction;
class QDockWindow;
class QLineEdit;
class RichtextView;
class FindDialog;


class MainWindow : public QMainWindow
{ 
    Q_OBJECT

public:
    MainWindow(QWidget *parent=0, const char *name=0);
    ~MainWindow();

private slots:
    void setActiveWindow();
    void gotoFile(const QString &url);
    void wentToFile(const QString &url);
    void fileOpen();
    void filePrint();
    void fileIndex();
    void editFind();
    void editIsearch();
    void editSettings();
    void isearchChanged();
    void isearchReturnPressed();
    void historyHighlighted(int id);
    void historyActivated(int id);
    void bookmarkHighlighted(int id);
    void bookmarkActivated(int id);
    void bookmarkAdd();
    void bookmarkRemove();
    void helpAbout();

private:
    virtual bool eventFilter(QObject *o, QEvent *e);
    void updateHistoryMenu();
    void updateBookmarkMenu();

    typedef QPair<QString,QString> Bookmark;
    
    QValueList<Bookmark> historyEntries;
    QValueList<Bookmark> bookmarks;
    QPopupMenu *historyMenu;
    QPopupMenu *bookmarkMenu;
    
    QDockWindow *dockWindow;
    RichtextView *htmlView;
    FindDialog *findDialog;
    QLineEdit *isearchEdit;
    QAction *fileOpenAction;
    QAction *filePrintAction;
    QAction *fileIndexAction;
    QAction *fileExitAction;
    QAction *editCopyAction;
    QAction *editFindAction;
    QAction *editIsearchAction;
    QAction *editSettingsAction;
    QAction *viewBackwardAction;
    QAction *viewForwardAction;
    QAction *viewZoomInAction;
    QAction *viewZoomOutAction;
    QAction *bookmarkAddAction;
    QAction *bookmarkRemoveAction;
    QAction *helpAboutAction;
};

#endif // _MAINWINDOW_H_
