/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef _RICHTEXTFACTORY_H_
#define _RICHTEXTFACTORY_H_

#include <qcache.h>
#include <qmime.h>
#include <qregexp.h>


class StringCache : public QCache<QString>
{
public:
    StringCache()
        : QCache<QString>()
    {
        setAutoDelete(true);
    }
    void insert(const QString &key, const QString &str)
    {
        QString *p = new QString(str);
        if (!QCache<QString>::insert(key, p, p->length()))
            delete p;
    }
};


class RichtextFactory : public QMimeSourceFactory
{
public:
    RichtextFactory();

    virtual QString makeAbsolute(const QString &name, const QString &context) const;
    virtual const QMimeSource *data(const QString &name) const;

private:
    QString makeAbsoluteUrl(const QString &url, const char *protocol) const;
    QString pageFromCommand(const QCString &name) const;
    QString manPage(const QString &name) const;
    QString aproposPage(const QString &name, bool apropos) const;
    QString infoPage(const QString &name) const;
    QString htdigPage(const QString &name) const;
    QString pydocPage(const QString &name) const;
    QString perlfaqPage(const QString &name) const;
    QString perldocPage(const QString &name) const;
    const QMimeSource *helpPage(const QString &name) const;
    
    mutable StringCache manCache;
    mutable StringCache htdigCache;
    mutable StringCache whatisCache;
    mutable StringCache aproposCache;
    mutable StringCache infoCache;
    mutable StringCache pydocCache;
    mutable StringCache perldocCache;
    mutable StringCache perlfaqCache;
    mutable StringCache helpCache;
    mutable QRegExp whatisRegExp;
};

#endif // _RICHTEXTFACTORY_H_
