/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@mail.berlios.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "searchwidget.h"
#include "global.h"
#include <qcombobox.h>
#include <qdir.h>
#include <qfile.h>
#include <qiconset.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qtoolbutton.h>
#include <qurl.h>


SearchWidget::SearchWidget(QWidget *parent, const char *name)
    : QVBox(parent, name)
{
    QIconSet enterIconSet = Global::iconSet(_("enter"));

    QWidget *htdigFrame = new QWidget(this);
    QLabel *htdigLabel = new QLabel(trUtf8("Search in HTML &docs:"), htdigFrame);
    htdigCombo = new QComboBox(true, htdigFrame);
    htdigCombo->setDuplicatesEnabled(false);
    QToolButton *htdigEnter = new QToolButton(htdigFrame);
    htdigEnter->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    htdigEnter->setIconSet(enterIconSet);

    QLabel *operatorLabel = new QLabel(trUtf8("Oper&ator:"), htdigFrame);
    operatorCombo = new QComboBox(htdigFrame);
    operatorCombo->insertItem(trUtf8("and"));
    operatorCombo->insertItem(trUtf8("or"));
    
    QLabel *sortLabel = new QLabel(trUtf8("So&rt:"), htdigFrame);
    sortCombo = new QComboBox(htdigFrame);
    sortCombo->insertItem(trUtf8("score"));
    sortCombo->insertItem(trUtf8("title"));
    sortCombo->insertItem(trUtf8("date"));

    QGridLayout *htdigLayout = new QGridLayout(htdigFrame, 4, 5, 4, 4);
    htdigLayout->addColSpacing(3, 3);
    htdigLayout->setColStretch(2, 2);
    htdigLayout->setColStretch(1, 2);
    htdigLayout->addMultiCellWidget(htdigLabel, 0, 0, 0, 2);
    htdigLayout->addMultiCellWidget(htdigCombo, 1, 1, 0, 2);
    htdigLayout->addWidget(htdigEnter, 1, 4);
    htdigLayout->addWidget(operatorLabel, 2, 0);
    htdigLayout->addWidget(operatorCombo, 2, 2);
    htdigLayout->addWidget(sortLabel, 3, 0);
    htdigLayout->addWidget(sortCombo, 3, 2);

    QFrame *htdigSep = new QFrame(this);
    htdigSep->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    htdigSep->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
    htdigSep->setFixedHeight(16);

    QWidget *manpageFrame = new QWidget(this);
    QLabel *manpageLabel = new QLabel(trUtf8("Search in &Man pages:"), manpageFrame);
    manpageCombo = new QComboBox(true, manpageFrame);
    manpageCombo->setDuplicatesEnabled(false);
    QToolButton *manpageEnter = new QToolButton(manpageFrame);
    manpageEnter->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    manpageEnter->setIconSet(enterIconSet);

    QLabel *methodLabel = new QLabel(trUtf8("Me&thod:"), manpageFrame);
    methodCombo = new QComboBox(manpageFrame);
    methodCombo->insertItem(trUtf8("string"));
    methodCombo->insertItem(trUtf8("regexp"));
    methodCombo->insertItem(trUtf8("wildcard"));

    QLabel *scopeLabel = new QLabel(trUtf8("Sc&ope"), manpageFrame);
    scopeCombo = new QComboBox(manpageFrame);
    scopeCombo->insertItem(trUtf8("title"));
    scopeCombo->insertItem(trUtf8("body"));

    QGridLayout *manpageLayout = new QGridLayout(manpageFrame, 4, 5, 4, 4);
    manpageLayout->addColSpacing(2, 3);
    manpageLayout->setColStretch(1, 10);
    manpageLayout->setColStretch(2, 2);
    manpageLayout->setColStretch(1, 2);
    manpageLayout->addMultiCellWidget(manpageLabel, 0, 0, 0, 2);
    manpageLayout->addMultiCellWidget(manpageCombo, 1, 1, 0, 2);
    manpageLayout->addWidget(manpageEnter, 1, 4);
    manpageLayout->addWidget(methodLabel, 2, 0);
    manpageLayout->addWidget(methodCombo, 2, 2);
    manpageLayout->addWidget(scopeLabel, 3, 0);
    manpageLayout->addWidget(scopeCombo, 3, 2);

    QFrame *manpageSep = new QFrame(this);
    manpageSep->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    manpageSep->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
    manpageSep->setFixedHeight(16);

    QWidget *pydocFrame = new QWidget(this);
    QLabel *pydocLabel = new QLabel(trUtf8("Search P&ython identifier:"), pydocFrame);
    pydocCombo = new QComboBox(true, pydocFrame);
    pydocCombo->setDuplicatesEnabled(false);
    QToolButton *pydocEnter = new QToolButton(pydocFrame);
    pydocEnter->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    pydocEnter->setIconSet(enterIconSet);
    
    QGridLayout *pydocLayout = new QGridLayout(pydocFrame, 2, 3, 4, 4);
    pydocLayout->addColSpacing(1, 3);
    pydocLayout->setColStretch(0, 1);
    pydocLayout->setColStretch(1, 0);
    pydocLayout->setColStretch(2, 0);
    pydocLayout->addWidget(pydocLabel, 0, 0);
    pydocLayout->addWidget(pydocCombo, 1, 0);
    pydocLayout->addWidget(pydocEnter, 1, 2);

    QFrame *pydocSep = new QFrame(this);
    pydocSep->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    pydocSep->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
    pydocSep->setFixedHeight(16);

    QWidget *perldocFrame = new QWidget(this);
    QLabel *perldocLabel = new QLabel(trUtf8("Search &Perl identifier:"), perldocFrame);
    perldocCombo = new QComboBox(true, perldocFrame);
    perldocCombo->setDuplicatesEnabled(false);
    QToolButton *perldocEnter = new QToolButton(perldocFrame);
    perldocEnter->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    perldocEnter->setIconSet(enterIconSet);
    
    QGridLayout *perldocLayout = new QGridLayout(perldocFrame, 2, 3, 4, 4);
    perldocLayout->addColSpacing(1, 3);
    perldocLayout->setColStretch(0, 10);
    perldocLayout->setColStretch(1, 0);
    perldocLayout->setColStretch(2, 0);
    perldocLayout->addWidget(perldocLabel, 0, 0);
    perldocLayout->addWidget(perldocCombo, 1, 0);
    perldocLayout->addWidget(perldocEnter, 1, 2);

    QFrame *perldocSep = new QFrame(this);
    perldocSep->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    perldocSep->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
    perldocSep->setFixedHeight(16);

    QWidget *perlfaqFrame = new QWidget(this);
    QLabel *perlfaqLabel = new QLabel(trUtf8("Search in Perl FA&Q:"), perlfaqFrame);
    perlfaqCombo = new QComboBox(true, perlfaqFrame);
    perlfaqCombo->setDuplicatesEnabled(false);
    QToolButton *perlfaqEnter = new QToolButton(perlfaqFrame);
    perlfaqEnter->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    perlfaqEnter->setIconSet(enterIconSet);
    
    QGridLayout *perlfaqLayout = new QGridLayout(perlfaqFrame, 2, 3, 4, 4);
    perlfaqLayout->addColSpacing(1, 3);
    perlfaqLayout->setColStretch(0, 10);
    perlfaqLayout->setColStretch(1, 0);
    perlfaqLayout->setColStretch(2, 0);
    perlfaqLayout->addWidget(perlfaqLabel, 0, 0);
    perlfaqLayout->addWidget(perlfaqCombo, 1, 0);
    perlfaqLayout->addWidget(perlfaqEnter, 1, 2);

    QWidget *spacer = new QWidget(this);
    spacer->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));

    setFocusProxy(htdigCombo);
    
    htdigLabel->setBuddy(htdigCombo);
    operatorLabel->setBuddy(operatorCombo);
    sortLabel->setBuddy(sortCombo);
    manpageLabel->setBuddy(manpageCombo);
    methodLabel->setBuddy(methodCombo);
    scopeLabel->setBuddy(scopeCombo);
    methodLabel->setBuddy(methodCombo);
    pydocLabel->setBuddy(pydocCombo);
    perldocLabel->setBuddy(perldocCombo);
    perlfaqLabel->setBuddy(perlfaqCombo);
    
    connect( htdigCombo, SIGNAL(activated(int)),
             this, SLOT(htdigClicked()) );
    connect( htdigEnter, SIGNAL(clicked()),
             this, SLOT(htdigClicked()) );
    connect( manpageCombo, SIGNAL(activated(int)),
             this, SLOT(manpageClicked()) );
    connect( manpageEnter, SIGNAL(clicked()),
             this, SLOT(manpageClicked()) );
    connect( pydocCombo, SIGNAL(activated(int)),
             this, SLOT(pydocClicked()) );
    connect( pydocEnter, SIGNAL(clicked()),
             this, SLOT(pydocClicked()) );
    connect( perldocCombo, SIGNAL(activated(int)),
             this, SLOT(perldocClicked()) );
    connect( perldocEnter, SIGNAL(clicked()),
             this, SLOT(perldocClicked()) );
    connect( perlfaqCombo, SIGNAL(activated(int)),
             this, SLOT(perlfaqClicked()) );
    connect( perlfaqEnter, SIGNAL(clicked()),
             this, SLOT(perlfaqClicked()) );
}


SearchWidget::~SearchWidget()
{}


static QString encode(const QString &text)
{
    QString str = text.stripWhiteSpace();
    QUrl::encode(str);
    return str;
}


void SearchWidget::htdigClicked()
{
    QString conf = QDir::homeDirPath() + _("/.dox/htdig/htdig.conf");
    if (!QFile::exists(conf)) {
        QMessageBox::information(this, trUtf8("Search"),
                                 trUtf8("You have to create a fulltext search index first.\n"
                                        "See under File->Create index."));
        return;
    }
    
    QString url = _("htdig://");
    url += encode(htdigCombo->currentText());
    bool operatorOr = (operatorCombo->currentItem() == 1);
    url += _(operatorOr? "?operator=or" : "?operator=and");
    if (sortCombo->currentItem() == 1)
        url += _("&sort=title");
    else if (sortCombo->currentItem() == 2)
        url += _("&sort=date");
    else
        url += _("&sort=score");
    
    emit gotoFile(url);
}


void SearchWidget::manpageClicked()
{
    bool whatis = scopeCombo->currentItem() == 0;
    QString url = _(whatis? "whatis://" : "apropos://");
    url += encode(manpageCombo->currentText());
    if (methodCombo->currentItem() == 1)
        url += _("?method=regex");
    else if (methodCombo->currentItem() == 2)
        url += _("?method=wildcard");
    else
        url += _("?method=string");

    emit gotoFile(url);
}


void SearchWidget::pydocClicked()
{
    QString url = _("pydoc://");
    url += encode(pydocCombo->currentText());
    emit gotoFile(url);
}


void SearchWidget::perldocClicked()
{
    QString url = _("perldoc://");
    url += encode(perldocCombo->currentText());
    emit gotoFile(url);
}


void SearchWidget::perlfaqClicked()
{
    QString url = _("perlfaq://");
    url += encode(perlfaqCombo->currentText());
    emit gotoFile(url);
}
