/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann                                  *
 *   bernd@berlios.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2       *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
***************************************************************************/

#include "settingsdialog.h"
#include "global.h"
#include <qsettings.h>
#include <qspinbox.h>


SettingsDialog::SettingsDialog(QWidget *parent, const char *name)
    : SettingsDialogBase(parent, name)
{
    QSettings *settings = Global::settings();
    int manCacheSize     = settings->readNumEntry(_("Dox/Cache/Man"),     2000);
    int infoCacheSize    = settings->readNumEntry(_("Dox/Cache/Info"),    2000);
    int whatisCacheSize  = settings->readNumEntry(_("Dox/Cache/Whatis"),  50);
    int aproposCacheSize = settings->readNumEntry(_("Dox/Cache/Apropos"), 50);
    int htdigCacheSize   = settings->readNumEntry(_("Dox/Cache/Htdig"),   50);
    int pydocCacheSize   = settings->readNumEntry(_("Dox/Cache/Pydoc"),   2000);
    int perldocCacheSize = settings->readNumEntry(_("Dox/Cache/Perldoc"), 100);
    int perlfaqCacheSize = settings->readNumEntry(_("Dox/Cache/Perlfaq"), 100);

    manCacheSpinBox->setValue(manCacheSize);
    infoCacheSpinBox->setValue(infoCacheSize);
    whatisCacheSpinBox->setValue(whatisCacheSize);
    aproposCacheSpinBox->setValue(aproposCacheSize);
    htdigCacheSpinBox->setValue(htdigCacheSize);
    pydocCacheSpinBox->setValue(pydocCacheSize);
    perldocCacheSpinBox->setValue(perldocCacheSize);
    perlfaqCacheSpinBox->setValue(perlfaqCacheSize);
}


void SettingsDialog::done(int r)
{
    if (r == Accepted) {
        int manCacheSize     = manCacheSpinBox->value();
        int infoCacheSize    = infoCacheSpinBox->value();
        int whatisCacheSize  = whatisCacheSpinBox->value();
        int aproposCacheSize = aproposCacheSpinBox->value();
        int htdigCacheSize   = htdigCacheSpinBox->value();
        int pydocCacheSize   = pydocCacheSpinBox->value();
        int perldocCacheSize = perldocCacheSpinBox->value();
        int perlfaqCacheSize = perlfaqCacheSpinBox->value();
        
        QSettings *settings = Global::settings();
        settings->writeEntry(_("Dox/Cache/Man"),     manCacheSize);
        settings->writeEntry(_("Dox/Cache/Info"),    infoCacheSize);
        settings->writeEntry(_("Dox/Cache/Whatis"),  whatisCacheSize);
        settings->writeEntry(_("Dox/Cache/Apropos"), aproposCacheSize);
        settings->writeEntry(_("Dox/Cache/Htdig"),   htdigCacheSize);
        settings->writeEntry(_("Dox/Cache/Pydoc"),   pydocCacheSize);
        settings->writeEntry(_("Dox/Cache/Perldoc"), perldocCacheSize);
        settings->writeEntry(_("Dox/Cache/Perlfaq"), perlfaqCacheSize);
    }

    SettingsDialogBase::done(r);
}
